/**
* Copyright (c) Huawei Technologies Co., Ltd. 2020-2022. All rights reserved.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at

* http://www.apache.org/licenses/LICENSE-2.0

* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.

* File sample_process.cpp
* Description: handle acl resource
*/
#include <iostream>
#include <sys/timeb.h>
#include "CarParams.h"
#include "../include/classifyPreprocess.h"
#include "AclLiteApp.h"

using namespace std;

namespace {
uint32_t kModelWidth = 256;
uint32_t kModelHeight = 256;
}

ClassifyPreprocessThread::ClassifyPreprocessThread(aclrtRunMode& runMode,ModelInfo programinfo)
    :runMode_(runMode), modelWidth_(programinfo.clsifyInputWidth), modelHeight_(programinfo.clsifyInputHeight),programinfo_(programinfo)
{
}

ClassifyPreprocessThread::~ClassifyPreprocessThread()
{
    dvpp_.DestroyResource();
}

AclLiteError ClassifyPreprocessThread::Init()
{
    AclLiteError aclRet = dvpp_.Init();
    if (aclRet) {
        ACLLITE_LOG_ERROR("Dvpp init failed, error %d", aclRet);
        return ACLLITE_ERROR;
    }

    return ACLLITE_OK;
}

AclLiteError ClassifyPreprocessThread::Process(int msgId, shared_ptr<void> data)
{
    AclLiteError ret = ACLLITE_OK;
    switch (msgId) {
        case MSG_DETECT_POSTPROC_DATA:
            ret = MsgProcess(static_pointer_cast<ObjDetectDataMsg>(data));
            ret = MsgSend(static_pointer_cast<ObjDetectDataMsg>(data));
            break;
        default:
            ACLLITE_LOG_INFO("Classify Preprocess thread ignore msg %d", msgId);
            break;
    }

    return ret;
}

AclLiteError ClassifyPreprocessThread::Crop(vector<ObjInfo> &carImgs, ImageData &orgImg)
{
    static int cnt = 0;
    AclLiteError ret = ACLLITE_OK;
    for (int i = 0; i < carImgs.size(); i++) {
        ret = dvpp_.Crop(carImgs[i].cropedImgs, orgImg,
                         carImgs[i].rectangle.lt.x, carImgs[i].rectangle.lt.y,
                         carImgs[i].rectangle.rb.x, carImgs[i].rectangle.rb.y);
        if (ret) {
            ACLLITE_LOG_ERROR("Crop image failed, error: %d, orgImg width %d, "
                              "height %d, size %d, crop area (%d, %d) (%d, %d)",
                              ret, orgImg.width, orgImg.height,
                              carImgs[i].cropedImgs.size, carImgs[i].rectangle.lt.x,
                              carImgs[i].rectangle.lt.y, carImgs[i].rectangle.rb.x,
                              carImgs[i].rectangle.rb.y);
            return ACLLITE_ERROR;
        }
    }
    return ret;
}

AclLiteError ClassifyPreprocessThread::Resize(vector<ObjInfo> &carImgs)
{
    AclLiteError ret = ACLLITE_OK;
    ImageData padingImage;
//    uint32_t max_edge;

    for (size_t i = 0; i < carImgs.size(); i++) {
//        carImgs[i].cropedImgs.height = carImgs[i].cropedImgs.alignHeight;
//        carImgs[i].cropedImgs.width = carImgs[i].cropedImgs.alignWidth;
//        if ((carImgs[i].cropedImgs.height / carImgs[i].cropedImgs.width)>1.5){
//            ret = dvpp_.ProportionPasteCenter(carImgs[i].resizedImgs,carImgs[i].cropedImgs,kModelWidth, kModelHeight);
//        } else{
//            ret = dvpp_.Resize(carImgs[i].resizedImgs, carImgs[i].cropedImgs,
//                               kModelWidth, kModelHeight);
//        }
        ret = dvpp_.CropPaste(carImgs[i].resizedImgs,carImgs[i].cropedImgs, modelWidth_, modelHeight_,
                              0, 0, carImgs[i].cropedImgs.width,carImgs[i].cropedImgs.height);

//        if ((int) i ==4){
//        ret = CopyImageToLocal(padingImage, carImgs[i].resizedImgs, runMode_);
//
//
//        cv::Mat yuvimg(padingImage.alignHeight*1.5, padingImage.alignWidth, CV_8UC1, padingImage.data.get());
//        cv::Mat saveframe;
//        cv::cvtColor(yuvimg, saveframe, CV_YUV2BGR_NV12);
//        cv::imwrite("../data/pic/copyface" + to_string(i) +".jpg",saveframe);
//        }

        if (ret) {
            ACLLITE_LOG_ERROR("ColorRecognition Resize image failed");
            return ACLLITE_ERROR;
        }
    }
    return ret;
}

AclLiteError ClassifyPreprocessThread::MsgProcess(shared_ptr<ObjDetectDataMsg> objDetectDataMsg)
{
    if (objDetectDataMsg->isLastFrame == 1)
        return ACLLITE_OK;

    objDetectDataMsg->flag = 0;
    // No car detected
    if (objDetectDataMsg->objInfo.size() == 0) {
        objDetectDataMsg->flag = 1;
        return ACLLITE_OK;
    }

    ImageData imageDevice;
    AclLiteError ret = CopyImageToDevice(imageDevice, objDetectDataMsg->imageFrame, runMode_, MEMORY_DVPP);
    if (ret == ACLLITE_ERROR) {
        ACLLITE_LOG_ERROR("Copy image to device failed");
        return ACLLITE_ERROR;
    }

    // crop
    ret = Crop(objDetectDataMsg->objInfo, imageDevice);
    if (ret) {
        ACLLITE_LOG_ERROR("Crop all the data failed, all the data failed");
        return ACLLITE_ERROR;
    }
    // resize
    ret = Resize(objDetectDataMsg->objInfo);
    if (ret) {
        ACLLITE_LOG_ERROR("Resize all the data failed, all the data failed");
        return ACLLITE_ERROR;
    }
    return ACLLITE_OK;
}

AclLiteError ClassifyPreprocessThread::MsgSend(shared_ptr<ObjDetectDataMsg> objDetectDataMsg)
{
    while (1) {
        AclLiteError ret = SendMessage(objDetectDataMsg->inferThreadId, MSG_CLASSIFY_PREPROC_DATA, objDetectDataMsg);
        if (ret == ACLLITE_ERROR_ENQUEUE) {
            usleep(500);
            continue;
        } else if(ret == ACLLITE_OK) {
            break;
        } else {
            ACLLITE_LOG_ERROR("Send read frame message failed, error %d", ret);
            return ret;
        }
    }

    return ACLLITE_OK;
}
